@@ -1,10 +1,10 @@
-from abc import ABC, abstractmethod
+from abc import abstractmethod
 from typing import List, Optional, Dict
 
-from haystack import Document
+from haystack import Document, BaseComponent
 
 
-class BaseGenerator(ABC):
+class BaseGenerator(BaseComponent):
     """
     Abstract class for Generators
     """
@@ -23,12 +23,11 @@ class BaseGenerator(ABC):
         """
         pass
 
-    def run(self, query: str, documents: List[Document], top_k_generator: Optional[int] = None, **kwargs):
+    def run(self, query: str, documents: List[Document], top_k: Optional[int] = None): # type: ignore
 
         if documents:
-            results = self.predict(query=query, documents=documents, top_k=top_k_generator)
+            results = self.predict(query=query, documents=documents, top_k=top_k)
         else:
             results = {"answers": []}
 
-        results.update(**kwargs)
         return results, "output_1"
