@@ -5,7 +5,6 @@ from typing import List, Optional, Union, Dict, Any
 from collections import defaultdict
 from time import perf_counter
 
-import numpy as np
 from farm.data_handler.data_silo import DataSilo
 from farm.data_handler.processor import SquadProcessor
 from farm.data_handler.dataloader import NamedDataLoader
@@ -13,12 +12,10 @@ from farm.data_handler.inputs import QAInput, Question
 from farm.infer import QAInferencer
 from farm.modeling.optimization import initialize_optimizer
 from farm.modeling.predictions import QAPred, QACandidate
-from farm.modeling.adaptive_model import BaseAdaptiveModel, AdaptiveModel
+from farm.modeling.adaptive_model import AdaptiveModel
 from farm.train import Trainer
 from farm.eval import Evaluator
 from farm.utils import set_all_seeds, initialize_device_settings
-from scipy.special import expit
-import shutil
 
 from haystack import Document
 from haystack.document_store.base import BaseDocumentStore
@@ -41,22 +38,28 @@ class FARMReader(BaseReader):
     def __init__(
         self,
         model_name_or_path: Union[str, Path],
+        model_version: Optional[str] = None,
         context_window_size: int = 150,
         batch_size: int = 50,
         use_gpu: bool = True,
         no_ans_boost: float = 0.0,
         return_no_answer: bool = False,
+        top_k: int = 10,
         top_k_per_candidate: int = 3,
         top_k_per_sample: int = 1,
         num_processes: Optional[int] = None,
         max_seq_len: int = 256,
         doc_stride: int = 128,
+        progress_bar: bool = True,
+        duplicate_filtering: int = 0,
+        use_confidence_scores: bool = True
     ):
 
         """
         :param model_name_or_path: Directory of a saved model or the name of a public model e.g. 'bert-base-cased',
         'deepset/bert-base-cased-squad2', 'deepset/bert-base-cased-squad2', 'distilbert-base-uncased-distilled-squad'.
         See https://huggingface.co/models for full list of available models.
+        :param model_version: The version of model to use from the HuggingFace model hub. Can be tag name, branch name, or commit hash.
         :param context_window_size: The size, in characters, of the window around the answer span that is used when
                                     displaying the context around the answer.
         :param batch_size: Number of samples the model receives in one batch for inference.
@@ -68,6 +71,7 @@ class FARMReader(BaseReader):
         If a negative number, there is a lower chance of "no_answer" being predicted.
         If a positive number, there is an increased chance of "no_answer"
         :param return_no_answer: Whether to include no_answer predictions in the results.
+        :param top_k: The maximum number of answers to return
         :param top_k_per_candidate: How many answers to extract for each candidate doc that is coming from the retriever (might be a long text).
         Note that this is not the number of "final answers" you will receive
         (see `top_k` in FARMReader.predict() or Finder.get_answers() for that)
@@ -84,14 +88,29 @@ class FARMReader(BaseReader):
                               want to debug the Language Model, you might need to disable multiprocessing!
         :param max_seq_len: Max sequence length of one input text for the model
         :param doc_stride: Length of striding window for splitting long texts (used if ``len(text) > max_seq_len``)
-
+        :param progress_bar: Whether to show a tqdm progress bar or not.
+                             Can be helpful to disable in production deployments to keep the logs clean.
+        :param duplicate_filtering: Answers are filtered based on their position. Both start and end position of the answers are considered.
+                                    The higher the value, answers that are more apart are filtered out. 0 corresponds to exact duplicates. -1 turns off duplicate removal.
         """
 
+        # save init parameters to enable export of component config as YAML
+        self.set_config(
+            model_name_or_path=model_name_or_path, model_version=model_version, context_window_size=context_window_size,
+            batch_size=batch_size, use_gpu=use_gpu, no_ans_boost=no_ans_boost, return_no_answer=return_no_answer,
+            top_k=top_k, top_k_per_candidate=top_k_per_candidate, top_k_per_sample=top_k_per_sample,
+            num_processes=num_processes, max_seq_len=max_seq_len, doc_stride=doc_stride, progress_bar=progress_bar,
+            duplicate_filtering=duplicate_filtering, use_confidence_scores=use_confidence_scores
+        )
+
         self.return_no_answers = return_no_answer
+        self.top_k = top_k
         self.top_k_per_candidate = top_k_per_candidate
         self.inferencer = QAInferencer.load(model_name_or_path, batch_size=batch_size, gpu=use_gpu,
-                                          task_type="question_answering", max_seq_len=max_seq_len,
-                                          doc_stride=doc_stride, num_processes=num_processes)
+                                            task_type="question_answering", max_seq_len=max_seq_len,
+                                            doc_stride=doc_stride, num_processes=num_processes, revision=model_version,
+                                            disable_tqdm=not progress_bar,
+                                            strict=False)
         self.inferencer.model.prediction_heads[0].context_window_size = context_window_size
         self.inferencer.model.prediction_heads[0].no_ans_boost = no_ans_boost
         self.inferencer.model.prediction_heads[0].n_best = top_k_per_candidate + 1 # including possible no_answer
@@ -99,8 +118,15 @@ class FARMReader(BaseReader):
             self.inferencer.model.prediction_heads[0].n_best_per_sample = top_k_per_sample
         except:
             logger.warning("Could not set `top_k_per_sample` in FARM. Please update FARM version.")
+        try:
+            self.inferencer.model.prediction_heads[0].duplicate_filtering = duplicate_filtering
+        except:
+            logger.warning("Could not set `duplicate_filtering` in FARM. Please update FARM version.")
         self.max_seq_len = max_seq_len
         self.use_gpu = use_gpu
+        self.progress_bar = progress_bar
+        self.device, _ = initialize_device_settings(use_cuda=self.use_gpu)
+        self.use_confidence_scores = use_confidence_scores
 
     def train(
         self,
@@ -195,16 +221,9 @@ class FARMReader(BaseReader):
         # and calculates a few descriptive statistics of our datasets
         data_silo = DataSilo(processor=processor, batch_size=batch_size, distributed=False, max_processes=num_processes)
 
-        # Quick-fix until this is fixed upstream in FARM:
-        # We must avoid applying DataParallel twice (once when loading the inferencer,
-        # once when calling initalize_optimizer)
-        self.inferencer.model.save("tmp_model")
-        model = BaseAdaptiveModel.load(load_dir="tmp_model", device=device, strict=True)
-        shutil.rmtree('tmp_model')
-
         # 3. Create an optimizer and pass the already initialized model
         model, optimizer, lr_schedule = initialize_optimizer(
-            model=model,
+            model=self.inferencer.model,
             # model=self.inferencer.model,
             learning_rate=learning_rate,
             schedule_opts={"name": "LinearWarmup", "warmup_proportion": warmup_proportion},
@@ -224,6 +243,7 @@ class FARMReader(BaseReader):
             evaluate_every=evaluate_every,
             device=device,
             use_amp=use_amp,
+            disable_tqdm=not self.progress_bar
         )
 
 
@@ -268,7 +288,7 @@ class FARMReader(BaseReader):
         """
         Use loaded QA model to find answers for a list of queries in each query's supplied list of Document.
 
-        Returns list of dictionaries containing answers sorted by (desc.) probability
+        Returns list of dictionaries containing answers sorted by (desc.) score
 
         :param query_doc_list: List of dictionaries containing queries with their retrieved documents
         :param top_k: The maximum number of answers to return for each query
@@ -276,6 +296,8 @@ class FARMReader(BaseReader):
         :return: List of dictionaries containing query and answers
         """
 
+        if top_k is None:
+            top_k = self.top_k
         # convert input to FARM format
         inputs = []
         number_of_docs = []
@@ -297,7 +319,7 @@ class FARMReader(BaseReader):
         self.inferencer.batch_size = batch_size
         # make predictions on all document-query pairs
         predictions = self.inferencer.inference_from_objects(
-            objects=inputs, return_json=False, multiprocessing_chunksize=1
+            objects=inputs, return_json=False, multiprocessing_chunksize=10
         )
 
         # group predictions together
@@ -327,7 +349,7 @@ class FARMReader(BaseReader):
         """
         Use loaded QA model to find answers for a query in the supplied list of Document.
 
-        Returns dictionaries containing answers sorted by (desc.) probability.
+        Returns dictionaries containing answers sorted by (desc.) score.
         Example:
          ```python
             |{
@@ -337,8 +359,7 @@ class FARMReader(BaseReader):
             |                 'context': " She travels with her father, Eddard, to King's Landing when he is ",
             |                 'offset_answer_start': 147,
             |                 'offset_answer_end': 154,
-            |                 'probability': 0.9787139466668613,
-            |                 'score': None,
+            |                 'score': 0.9787139466668613,
             |                 'document_id': '1337'
             |                 },...
             |              ]
@@ -350,7 +371,8 @@ class FARMReader(BaseReader):
         :param top_k: The maximum number of answers to return
         :return: Dict containing query and answers
         """
-
+        if top_k is None:
+            top_k = self.top_k
         # convert input to FARM format
         inputs = []
         for doc in documents:
@@ -372,7 +394,7 @@ class FARMReader(BaseReader):
 
         return result
 
-    def eval_on_file(self, data_dir: str, test_filename: str, device: str):
+    def eval_on_file(self, data_dir: str, test_filename: str, device: Optional[str] = None):
         """
         Performs evaluation on a SQuAD-formatted file.
         Returns a dict containing the following metrics:
@@ -384,9 +406,11 @@ class FARMReader(BaseReader):
         :type data_dir: Path or str
         :param test_filename: The name of the file containing the test data in SQuAD format.
         :type test_filename: str
-        :param device: The device on which the tensors should be processed. Choose from "cpu" and "cuda".
+        :param device: The device on which the tensors should be processed. Choose from "cpu" and "cuda" or use the Reader's device by default.
         :type device: str
         """
+        if device is None:
+            device = self.device
         eval_processor = SquadProcessor(
             tokenizer=self.inferencer.processor.tokenizer,
             max_seq_len=self.inferencer.processor.max_seq_len,
@@ -415,10 +439,11 @@ class FARMReader(BaseReader):
     def eval(
             self,
             document_store: BaseDocumentStore,
-            device: str,
+            device: Optional[str] = None,
             label_index: str = "label",
             doc_index: str = "eval_document",
             label_origin: str = "gold_label",
+            calibrate_conf_scores: bool = False
     ):
         """
         Performs evaluation on evaluation documents in the DocumentStore.
@@ -428,11 +453,14 @@ class FARMReader(BaseReader):
               - "top_n_accuracy": Proportion of predicted answers that overlap with correct answer
 
         :param document_store: DocumentStore containing the evaluation documents
-        :param device: The device on which the tensors should be processed. Choose from "cpu" and "cuda".
+        :param device: The device on which the tensors should be processed. Choose from "cpu" and "cuda" or use the Reader's device by default.
         :param label_index: Index/Table name where labeled questions are stored
         :param doc_index: Index/Table name where documents that are used for evaluation are stored
+        :param label_origin: Field name where the gold labels are stored
+        :param calibrate_conf_scores: Whether to calibrate the temperature for temperature scaling of the confidence scores
         """
-
+        if device is None:
+            device = self.device
         if self.top_k_per_candidate != 4:
             logger.info(f"Performing Evaluation using top_k_per_candidate = {self.top_k_per_candidate} \n"
                         f"and consequently, QuestionAnsweringPredictionHead.n_best = {self.top_k_per_candidate + 1}. \n"
@@ -463,33 +491,35 @@ class FARMReader(BaseReader):
                 "context": doc.text
             }
             # get all questions / answers
-            aggregated_per_question: Dict[str, Any] = defaultdict(list)
+            aggregated_per_question: Dict[tuple, Any] = defaultdict(list)
+            id_question_tuple = (label.id, label.question)
             if doc_id in aggregated_per_doc:
                 for label in aggregated_per_doc[doc_id]:
                     # add to existing answers
-                    if label.question in aggregated_per_question.keys():
+                    if id_question_tuple in aggregated_per_question.keys():
                         if label.offset_start_in_doc == 0 and label.answer == "":
                             continue
                         else:
                             # Hack to fix problem where duplicate questions are merged by doc_store processing creating a QA example with 8 annotations > 6 annotation max
-                            if len(aggregated_per_question[label.question]["answers"]) >= 6:
+                            if len(aggregated_per_question[id_question_tuple]["answers"]) >= 6:
+                                logger.warning(f"Answers in this sample are being dropped because it has more than 6 answers. (doc_id: {doc_id}, question: {label.question}, label_id: {label.id})")
                                 continue
-                            aggregated_per_question[label.question]["answers"].append({
+                            aggregated_per_question[id_question_tuple]["answers"].append({
                                         "text": label.answer,
                                         "answer_start": label.offset_start_in_doc})
-                            aggregated_per_question[label.question]["is_impossible"] = False
+                            aggregated_per_question[id_question_tuple]["is_impossible"] = False
                     # create new one
                     else:
                         # We don't need to create an answer dict if is_impossible / no_answer
                         if label.offset_start_in_doc == 0 and label.answer == "":
-                            aggregated_per_question[label.question] = {
+                            aggregated_per_question[id_question_tuple] = {
                                 "id": str(hash(str(doc_id) + label.question)),
                                 "question": label.question,
                                 "answers": [],
                                 "is_impossible": True
                             }
                         else:
-                            aggregated_per_question[label.question] = {
+                            aggregated_per_question[id_question_tuple] = {
                                 "id": str(hash(str(doc_id)+label.question)),
                                 "question": label.question,
                                 "answers": [{
@@ -513,13 +543,13 @@ class FARMReader(BaseReader):
 
         evaluator = Evaluator(data_loader=data_loader, tasks=self.inferencer.processor.tasks, device=device)
 
-        eval_results = evaluator.eval(self.inferencer.model)
+        eval_results = evaluator.eval(self.inferencer.model, calibrate_conf_scores=calibrate_conf_scores)
         toc = perf_counter()
         reader_time = toc - tic
         results = {
-            "EM": eval_results[0]["EM"],
-            "f1": eval_results[0]["f1"],
-            "top_n_accuracy": eval_results[0]["top_n_accuracy"],
+            "EM": eval_results[0]["EM"] * 100,
+            "f1": eval_results[0]["f1"] * 100,
+            "top_n_accuracy": eval_results[0]["top_n_accuracy"] * 100,
             "top_n": self.inferencer.model.prediction_heads[0].n_best,
             "reader_time": reader_time,
             "seconds_per_query": reader_time / n_queries
@@ -544,9 +574,7 @@ class FARMReader(BaseReader):
                 else:
                     cur = {
                         "answer": ans.answer,
-                        "score": ans.score,
-                        # just a pseudo prob for now
-                        "probability": self._get_pseudo_prob(ans.score),
+                        "score": ans.confidence if self.use_confidence_scores else ans.score,
                         "context": ans.context_window,
                         "offset_start": ans.offset_answer_start - ans.offset_context_window_start,
                         "offset_end": ans.offset_answer_end - ans.offset_context_window_start,
@@ -563,7 +591,7 @@ class FARMReader(BaseReader):
             answers += answers_per_document[:self.top_k_per_candidate]
 
         # calculate the score for predicting 'no answer', relative to our best positive answer score
-        no_ans_prediction, max_no_ans_gap = self._calc_no_answer(no_ans_gaps, best_score_answer)
+        no_ans_prediction, max_no_ans_gap = self._calc_no_answer(no_ans_gaps, best_score_answer, self.use_confidence_scores)
         if self.return_no_answers:
             answers.append(no_ans_prediction)
 
@@ -573,9 +601,31 @@ class FARMReader(BaseReader):
 
         return answers, max_no_ans_gap
 
-    @staticmethod
-    def _get_pseudo_prob(score: float):
-        return float(expit(np.asarray(score) / 8))
+    def calibrate_confidence_scores(
+            self,
+            document_store: BaseDocumentStore,
+            device: Optional[str] = None,
+            label_index: str = "label",
+            doc_index: str = "eval_document",
+            label_origin: str = "gold_label"
+    ):
+        """
+        Calibrates confidence scores on evaluation documents in the DocumentStore.
+
+        :param document_store: DocumentStore containing the evaluation documents
+        :param device: The device on which the tensors should be processed. Choose from "cpu" and "cuda" or use the Reader's device by default.
+        :param label_index: Index/Table name where labeled questions are stored
+        :param doc_index: Index/Table name where documents that are used for evaluation are stored
+        :param label_origin: Field name where the gold labels are stored
+        """
+        if device is None:
+            device = self.device
+        self.eval(document_store=document_store,
+                  device=device,
+                  label_index=label_index,
+                  doc_index=doc_index,
+                  label_origin=label_origin,
+                  calibrate_conf_scores=True)
 
     @staticmethod
     def _check_no_answer(c: QACandidate):
@@ -591,7 +641,7 @@ class FARMReader(BaseReader):
     def predict_on_texts(self, question: str, texts: List[str], top_k: Optional[int] = None):
         """
         Use loaded QA model to find answers for a question in the supplied list of Document.
-        Returns dictionaries containing answers sorted by (desc.) probability.
+        Returns dictionaries containing answers sorted by (desc.) score.
         Example:
          ```python
             |{
@@ -601,8 +651,7 @@ class FARMReader(BaseReader):
             |                 'context': " She travels with her father, Eddard, to King's Landing when he is ",
             |                 'offset_answer_start': 147,
             |                 'offset_answer_end': 154,
-            |                 'probability': 0.9787139466668613,
-            |                 'score': None,
+            |                 'score': 0.9787139466668613,
             |                 'document_id': '1337'
             |                 },...
             |              ]
