@@ -17,9 +17,11 @@ class TransformersReader(BaseReader):
     def __init__(
         self,
         model_name_or_path: str = "distilbert-base-uncased-distilled-squad",
+        model_version: Optional[str] = None,
         tokenizer: Optional[str] = None,
         context_window_size: int = 70,
         use_gpu: int = 0,
+        top_k: int = 10,
         top_k_per_candidate: int = 4,
         return_no_answers: bool = True,
         max_seq_len: int = 256,
@@ -38,10 +40,12 @@ class TransformersReader(BaseReader):
         :param model_name_or_path: Directory of a saved model or the name of a public model e.g. 'bert-base-cased',
         'deepset/bert-base-cased-squad2', 'deepset/bert-base-cased-squad2', 'distilbert-base-uncased-distilled-squad'.
         See https://huggingface.co/models for full list of available models.
+        :param model_version: The version of model to use from the HuggingFace model hub. Can be tag name, branch name, or commit hash.
         :param tokenizer: Name of the tokenizer (usually the same as model)
         :param context_window_size: Num of chars (before and after the answer) to return as "context" for each answer.
                                     The context usually helps users to understand if the answer really makes sense.
         :param use_gpu: If < 0, then use cpu. If >= 0, this is the ordinal of the gpu to use
+        :param top_k: The maximum number of answers to return
         :param top_k_per_candidate: How many answers to extract for each candidate doc that is coming from the retriever (might be a long text).
         Note that this is not the number of "final answers" you will receive
         (see `top_k` in TransformersReader.predict() or Finder.get_answers() for that)
@@ -53,8 +57,17 @@ class TransformersReader(BaseReader):
         :param doc_stride: length of striding window for splitting long texts (used if len(text) > max_seq_len)
 
         """
-        self.model = pipeline('question-answering', model=model_name_or_path, tokenizer=tokenizer, device=use_gpu)
+
+        # save init parameters to enable export of component config as YAML
+        self.set_config(
+            model_name_or_path=model_name_or_path, model_version=model_version, tokenizer=tokenizer,
+            context_window_size=context_window_size, use_gpu=use_gpu, top_k=top_k, doc_stride=doc_stride,
+            top_k_per_candidate=top_k_per_candidate, return_no_answers=return_no_answers, max_seq_len=max_seq_len,
+        )
+
+        self.model = pipeline('question-answering', model=model_name_or_path, tokenizer=tokenizer, device=use_gpu, revision=model_version)
         self.context_window_size = context_window_size
+        self.top_k = top_k
         self.top_k_per_candidate = top_k_per_candidate
         self.return_no_answers = return_no_answers
         self.max_seq_len = max_seq_len
@@ -66,7 +79,7 @@ class TransformersReader(BaseReader):
         """
         Use loaded QA model to find answers for a query in the supplied list of Document.
 
-        Returns dictionaries containing answers sorted by (desc.) probability.
+        Returns dictionaries containing answers sorted by (desc.) score.
         Example:
 
          ```python
@@ -77,8 +90,7 @@ class TransformersReader(BaseReader):
             |                 'context': " She travels with her father, Eddard, to King's Landing when he is ",
             |                 'offset_answer_start': 147,
             |                 'offset_answer_end': 154,
-            |                 'probability': 0.9787139466668613,
-            |                 'score': None,
+            |                 'score': 0.9787139466668613,
             |                 'document_id': '1337'
             |                 },...
             |              ]
@@ -91,6 +103,8 @@ class TransformersReader(BaseReader):
         :return: Dict containing query and answers
 
         """
+        if top_k is None:
+            top_k = self.top_k
         # get top-answers for each candidate passage
         answers = []
         no_ans_gaps = []
@@ -122,8 +136,7 @@ class TransformersReader(BaseReader):
                         "context": doc.text[context_start:context_end],
                         "offset_start": pred["start"],
                         "offset_end": pred["end"],
-                        "probability": pred["score"],
-                        "score": None,
+                        "score": pred["score"],
                         "document_id": doc.id,
                         "meta": doc.meta
                     })
@@ -140,9 +153,9 @@ class TransformersReader(BaseReader):
 
         if self.return_no_answers:
             answers.append(no_ans_prediction)
-        # sort answers by their `probability` and select top-k
+        # sort answers by their `score` and select top-k
         answers = sorted(
-            answers, key=lambda k: k["probability"], reverse=True
+            answers, key=lambda k: k["score"], reverse=True
         )
         answers = answers[:top_k]
 
