@@ -1,10 +1,10 @@
-from abc import ABC, abstractmethod
-from typing import List, Optional, Dict
+from abc import abstractmethod
+from typing import List, Dict, Optional
 
-from haystack import Document
+from haystack import Document, BaseComponent
 
 
-class BaseSummarizer(ABC):
+class BaseSummarizer(BaseComponent):
     """
     Abstract class for Summarizer
     """
@@ -12,7 +12,7 @@ class BaseSummarizer(ABC):
     outgoing_edges = 1
 
     @abstractmethod
-    def predict(self, documents: List[Document], generate_single_summary: bool = False) -> List[Document]:
+    def predict(self, documents: List[Document], generate_single_summary: Optional[bool] = None) -> List[Document]:
         """
         Abstract method for creating a summary.
 
@@ -26,12 +26,9 @@ class BaseSummarizer(ABC):
         """
         pass
 
-    def run(self, documents: List[Document], generate_single_summary: bool = False, **kwargs):
+    def run(self, documents: List[Document], generate_single_summary: Optional[bool] = None): # type: ignore
 
-        results: Dict = {
-            "documents": [],
-            **kwargs
-        }
+        results: Dict = {"documents": []}
 
         if documents:
             results["documents"] = self.predict(documents=documents, generate_single_summary=generate_single_summary)
